/*
 *  Copyright (C) 2002-2006  The DOSBox Team
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* $Id: serialport.h,v 1.12 2006/02/09 11:47:48 qbix79 Exp $ */

#ifndef DOSBOX_PARPORT_H
#define DOSBOX_PARPORT_H

// Uncomment this for a lot of debug messages:
// #define PARALLELPORT_DEBUGMSG 

#ifndef DOSBOX_DOSBOX_H
#include "dosbox.h"
#endif
#ifndef DOSBOX_INOUT_H
#include "inout.h"
#endif
#ifndef DOSBOX_TIMER_H
#include "timer.h"
#endif

#include "dos_inc.h"

class device_LPT : public DOS_Device {
public:
	// Creates a LPT device that communicates with the num-th parallel port, i.e. is LPTnum
	device_LPT(Bit8u num, class CParallel* pp);
	~device_LPT();
	bool Read(Bit8u * data,Bit16u * size);
	bool Write(Bit8u * data,Bit16u * size);
	bool Seek(Bit32u * pos,Bit32u type);
	bool Close();
	Bit16u GetInformation(void);
private:
	CParallel* pportclass;
	Bit8u num; // This device is LPTnum
};


class CParallel {
public:

	// Constructor takes base port (, etc.), IRQ, and initial bps //
	CParallel(IO_ReadHandler* rh, IO_WriteHandler* wh,
			TIMER_TickHandler TimerHandler, 
			Bit16u initbase, Bit8u initirq);
	
	TIMER_TickHandler TimerHnd;
	virtual ~CParallel();
	void InstallTimerHandler(TIMER_TickHandler);
	
	IO_ReadHandleObject ReadHandler[3];
	IO_WriteHandleObject WriteHandler[3];

	void Timer(void);
	virtual void Timer2(void)=0;
	
	Bitu base;
	Bitu irq;
	
	// read data line register
	virtual Bitu Read_PR()=0;
	virtual Bitu Read_COM()=0;
	virtual Bitu Read_SR()=0;

	virtual void Write_PR(Bitu)=0;
	virtual void Write_CON(Bitu)=0;
	virtual void Write_IOSEL(Bitu)=0;

	void Write_reserved(Bit8u data, Bit8u address);

	virtual bool Putchar(Bit8u)=0;
	bool Putchar_default(Bit8u);
	Bit8u getPrinterStatus();
	void initialize();
	


private:
	 DOS_Device* mydosdevice;
		
};


extern CParallel ** parallelPortObjects[3];

#define LPT1_BASE 0x378
#define LPT2_BASE 0x278
#define LPT3_BASE 0x3bc

#endif

